@@ -1,3 +1,4 @@
+# typed: false
 # coding: utf-8
 
 describe PDF::Reader::Buffer, "token method" do
@@ -211,6 +212,15 @@ describe PDF::Reader::Buffer, "token method" do
     end
   end
 
+  context "string with an escaped, unbalanced right param correctly" do
+    it "tokenises correctly" do
+      buf = parse_string("(James Code Monkey\\))")
+      expect(buf.token).to eql("(")
+      expect(buf.token).to eql("James Code Monkey\\)")
+      expect(buf.token).to eql(")")
+    end
+  end
+
   context "indirect reference" do
     it "tokenises correctly" do
       buf = parse_string("aaa 1 0 R bbb")
@@ -478,6 +488,19 @@ describe PDF::Reader::Buffer, "token method" do
     end
   end
 
+  context "inline image without a trailing 'EI'" do
+    context "inside a content stream" do
+      it "tokenises correctly" do
+        io = StringIO.new(binary_string("BI ID aaa bbb ccc \xF0\xF0\xF0"))
+        buf = PDF::Reader::Buffer.new(io, :content_stream => true)
+        expect(buf.pos).to eql(0)
+        expect {
+          buf.token
+        }.to raise_error(PDF::Reader::MalformedPDFError, "EI terminator not found")
+      end
+    end
+  end
+
   context "dict that has ID as a key" do
     it "tokenises correctly" do
       io = StringIO.new("<</ID /S1 >> BDC")
@@ -662,7 +685,12 @@ describe PDF::Reader::Buffer, "read method" do
 
   context "with a multi-line buffer (three \\n)" do
     context "without :skip_eol" do
-      it "returns raw data from the underlying IO"
+      it "returns raw data from the underlying IO" do
+        buf = parse_string("stream\n\n\nbbb")
+
+        expect(buf.token).to eql("stream")
+        expect(buf.read(5)).to eql("\n\nbbb")
+      end
     end
     context "with :skip_eol" do
       it "returns raw data from the underlying IO" do
@@ -676,7 +704,12 @@ describe PDF::Reader::Buffer, "read method" do
 
   context "with a multi-line buffer (\\r\\n)" do
     context "without :skip_eol" do
-      it "returns raw data from the underlying IO"
+      it "returns raw data from the underlying IO" do
+        buf = parse_string("stream\r\nbbb")
+
+        expect(buf.token).to eql("stream")
+        expect(buf.read(4)).to eql("\nbbb")
+      end
     end
     context "with :skip_eol" do
       it "returns raw data from the underlying IO" do
