@@ -1,3 +1,4 @@
+# typed: false
 # coding: utf-8
 
 # These specs are a kind of integration spec. They're not unit testing small pieces
@@ -13,12 +14,79 @@ describe PDF::Reader, "integration specs" do
   context "cairo-unicode-short" do
     let(:filename) { pdf_spec_file("cairo-unicode-short") }
 
-    it "interprets unicode strings correctly" do
+    it "extracts unicode strings correctly" do
       PDF::Reader.open(filename) do |reader|
         page = reader.page(1)
         expect(page.text).to eql("Chunky Bacon")
       end
     end
+
+    it "extracts unicode strings correctly from part of the page" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(
+          page.text(rect: PDF::Reader::Rectangle.new(29, 779, 100, 800))
+        ).to eql("Chunky")
+      end
+    end
+
+    # This spec assumes high precision in our glyph positioning calcualtions. It's likely
+    # that the specific x,y co-ords tested here might change a bit over time as small bugs
+    # in glyph positioning are ironed out (kerning, etc). I still think this test is worthwhile,
+    # to confirm the positions don't change unintentionally.
+    #
+    # pitstop co-ords
+    # C 32.0176, 779.5879
+    # h 48.8279, 779.89
+    # u 60.2067, 779.5879
+    # n 73.8355, 779.89
+    # k 87.3393, 779.89
+    # y 97.3745, 775.4526
+    it "extracts individual chars with correct positions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        some_runs = page.runs(
+          merge: false,
+          rect: PDF::Reader::Rectangle.new(29, 779, 100, 800)
+        )
+        expect(some_runs.size).to eql(6)
+        expect(some_runs[0].text).to eql("C")
+        expect(some_runs[0].origin).to be_close_to(PDF::Reader::Point.new(30,779.89))
+        expect(some_runs[1].text).to eql("h")
+        expect(some_runs[1].origin).to be_close_to(PDF::Reader::Point.new(44.89,779.89))
+        expect(some_runs[2].text).to eql("u")
+        expect(some_runs[2].origin).to be_close_to(PDF::Reader::Point.new(58.39, 779.89))
+        expect(some_runs[3].text).to eql("n")
+        expect(some_runs[3].origin).to be_close_to(PDF::Reader::Point.new(71.89, 779.89))
+        expect(some_runs[4].text).to eql("k")
+        expect(some_runs[4].origin).to be_close_to(PDF::Reader::Point.new(85.40, 779.89))
+        expect(some_runs[5].text).to eql("y")
+        expect(some_runs[5].origin).to be_close_to(PDF::Reader::Point.new(96.73, 779.89))
+      end
+    end
+
+    it "is portrait orientation" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.orientation).to eql("portrait")
+      end
+    end
+
+    it "returns correct page dimensions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        # A4 portrait
+        expect(page.width).to be_within(0.1).of(595.28)
+        expect(page.height).to be_within(0.1).of(841.89)
+      end
+    end
+
+    it "returns correct origin" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.origin).to eq PDF::Reader::Point.new(0,0)
+      end
+    end
   end
 
   context "vertical-text-in-identity-v" do
@@ -52,7 +120,9 @@ describe PDF::Reader, "integration specs" do
         expect(reader.pages.size).to eql(3)
 
         page = reader.page(1)
-        expect(page.text).to include("Dit\302\240is\302\240een\302\240pdf\302\240test\302\240van\302\240drie\302\240pagina")
+        expect(page.text).to include(
+          "Dit\302\240is\302\240een\302\240pdf\302\240test\302\240van\302\240drie\302\240pagina"
+        )
         expect(page.text).to include("’s")
         expect(page.text).to include("Pagina\302\2401")
       end
@@ -92,6 +162,17 @@ describe PDF::Reader, "integration specs" do
     end
   end
 
+  context "PDF with a content stream that is enclosed with CR characters only" do
+    let(:filename) { pdf_spec_file("content_stream_cr_only") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq("This is a weird PDF file")
+      end
+    end
+  end
+
   context "PDF with a content stream that is missing an operator (has hanging params)" do
     let(:filename) { pdf_spec_file("content_stream_missing_final_operator") }
 
@@ -115,20 +196,6 @@ describe PDF::Reader, "integration specs" do
     end
   end
 
-  context "PDF that has a content stream with a broken string" do
-    let(:filename) { pdf_spec_file("broken_string") }
-
-    # this file used to get us into a hard, endless loop. Make sure that doesn't still happen
-    it "doesn't hang when extracting doc info" do
-      Timeout::timeout(3) do
-        expect {
-          reader = PDF::Reader.new(filename)
-          reader.info
-        }.to raise_error(PDF::Reader::MalformedPDFError)
-      end
-    end
-  end
-
   context "PDF with a stream that has its length specified as an indirect reference" do
     let(:filename) { pdf_spec_file("content_stream_with_length_as_ref") }
 
@@ -142,7 +209,7 @@ describe PDF::Reader, "integration specs" do
   # PDF::Reader::XRef#object was saving an incorrect position when seeking. We
   # were saving the current pos of the underlying IO stream, then seeking back
   # to it. This was fine, except when there was still content in the buffer.
-  context "PDF with a stream that has its length specified as an indirect reference and uses windows line breaks" do
+  context "PDF with a stream length specified via indirect object and uses windows line breaks" do
     let(:filename) { pdf_spec_file("content_stream_with_length_as_ref_and_windows_breaks") }
 
     it "extracts text correctly" do
@@ -152,31 +219,19 @@ describe PDF::Reader, "integration specs" do
     end
   end
 
-  context "PDF has a content stream refers to a non-existant font" do
-    let(:filename) { pdf_spec_file("content_stream_refers_to_invalid_font") }
-
-    it "raises an exception" do
-      expect {
-        reader = PDF::Reader.new(filename)
-        reader.page(1).text
-      }.to raise_error(PDF::Reader::MalformedPDFError)
-    end
-  end
-
-  context "Empty file" do
-    it "raises an exception" do
-      expect {
-        PDF::Reader.new(StringIO.new(""))
-      }.to raise_error(PDF::Reader::MalformedPDFError)
-    end
-  end
-
   context "PDF that uses an ASCII85Decode filter" do
     let(:filename) { pdf_spec_file("ascii85_filter") }
 
+    # The text on this page is rotated 45 degrees, so mapping it to plain text isn't straight
+    # forward. Still, it'd be nice to see if this can be improved somehow
+    #
+    # If not, maybe it'd be good to drop this spec, and replace it with one that confirms the text
+    # position if a few key characters
     it "extracts text correctly" do
       PDF::Reader.open(filename) do |reader|
-        expect(reader.page(1).text).to match(/Et Iunia se/)
+        lines = reader.page(1).text.split("\n")
+        expect(lines[0].strip).to eq("E")
+        expect(lines[1].strip).to eq("t Iu")
       end
     end
   end
@@ -191,6 +246,16 @@ describe PDF::Reader, "integration specs" do
     end
   end
 
+  context "PDF that has dummy inline data no white-space before EI" do
+    let(:filename) { pdf_spec_file("inline_data_followed_by_ei") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        expect(reader.page(1).text).to eql("ID followed by EI on same line\n__END__")
+      end
+    end
+  end
+
   context "PDF that uses Form XObjects to repeat content" do
     let(:filename) { pdf_spec_file("form_xobject") }
 
@@ -249,9 +314,11 @@ describe PDF::Reader, "integration specs" do
   context "PDF that has a single space after the EOF marker" do
     let(:filename) { pdf_spec_file("space_after_eof") }
 
+    # This text is rotated at 45 degrees, which isn't particularly easy to map to plain text
+    # It would be nice if all the "Hello World" characters were at least visible though
     it "extracts text correctly" do
       PDF::Reader.open(filename) do |reader|
-        expect(reader.page(1).text).to eql("Hello World")
+        expect(reader.page(1).text).to eql("    r\n loo\nHe")
       end
     end
   end
@@ -727,22 +794,45 @@ describe PDF::Reader, "integration specs" do
     context "with the user pass" do
       let(:pass) { "apples" }
 
-      # TODO: remove this spec
-      it "raises UnsupportedFeatureError" do
-        expect {
-          PDF::Reader.open(filename, :password => pass) do |reader|
-            reader.page(1).text
-          end
-        }.to raise_error(PDF::Reader::EncryptedPDFError)
+      it "correctly extracts text" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.page(1).text).to start_with(
+            "This sample file is encrypted with a user password"
+          )
+        end
+      end
+      it "correctly extracts info" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.info).to eq(
+            :Creator=>"Writer",
+            :Producer=>"LibreOffice 3.3",
+            :CreationDate=>"D:20110814231057+10'00'",
+            :ModDate=>"D:20170115224358+11'00'"
+          )
+        end
       end
-
-      it "correctly extracts text"
-      it "correctly extracts info"
     end
 
     context "with the owner pass" do
-      it "correctly extracts text"
-      it "correctly extracts info"
+      let(:pass) { "password" }
+
+      it "correctly extracts text" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.page(1).text).to start_with(
+            "This sample file is encrypted with a user password"
+          )
+        end
+      end
+      it "correctly extracts info" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.info).to eq(
+            :Creator=>"Writer",
+            :Producer=>"LibreOffice 3.3",
+            :CreationDate=>"D:20110814231057+10'00'",
+            :ModDate=>"D:20170115224358+11'00'"
+          )
+        end
+      end
     end
   end
 
@@ -754,22 +844,64 @@ describe PDF::Reader, "integration specs" do
     context "with the user pass" do
       let(:pass) { "apples" }
 
-      # TODO: remove this spec
-      it "raises UnsupportedFeatureError" do
-        expect {
-          PDF::Reader.open(filename, :password => pass) do |reader|
-            reader.page(1).text
-          end
-        }.to raise_error(PDF::Reader::EncryptedPDFError)
+      it "correctly extracts text" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.page(1).text).to start_with(
+            "This sample file is encrypted with a user password"
+          )
+        end
       end
 
-      it "correctly extracts text"
-      it "correctly extracts info"
+      it "correctly extracts info" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.info).to eq(
+            :Creator=>"Writer",
+            :Producer=>"LibreOffice 3.3",
+            :CreationDate=>"D:20110814231057+10'00'",
+            :ModDate=>"D:20170115224526+11'00'"
+          )
+        end
+      end
     end
 
     context "with the owner pass" do
-      it "correctly extracts text"
-      it "correctly extracts info"
+      let(:pass) { "password" }
+
+      it "correctly extracts text" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.page(1).text).to start_with(
+            "This sample file is encrypted with a user password"
+          )
+        end
+      end
+      it "correctly extracts info" do
+        PDF::Reader.open(filename, :password => pass) do |reader|
+          expect(reader.info).to eq(
+            :Creator=>"Writer",
+            :Producer=>"LibreOffice 3.3",
+            :CreationDate=>"D:20110814231057+10'00'",
+            :ModDate=>"D:20170115224526+11'00'"
+          )
+        end
+      end
+    end
+  end
+
+  context "Encrypted PDF with an xref stream" do
+    let(:filename) {
+      pdf_spec_file("encrypted_and_xref_stream")
+    }
+
+    it "correctly extracts text" do
+      PDF::Reader.open(filename) do |reader|
+        expect(reader.page(1).text).to eq("This text is encrypted")
+      end
+    end
+
+    it "correctly parses indirect objects" do
+      PDF::Reader.open(filename) do |reader|
+        expect { reader.objects.values }.not_to raise_error
+      end
     end
   end
 
@@ -789,7 +921,10 @@ describe PDF::Reader, "integration specs" do
       expect(callbacks[0]).to eql(:name => :begin_inline_image, :args => [])
 
       # the second with the image header (colorspace, etc)
-      expect(callbacks[1]).to eql(:name => :begin_inline_image_data, :args => [:CS, :RGB, :I, true, :W, 234, :H, 70, :BPC, 8])
+      expect(callbacks[1]).to eql(
+        :name => :begin_inline_image_data,
+        :args => [:CS, :RGB, :I, true, :W, 234, :H, 70, :BPC, 8]
+      )
 
       # the last with the image data
       expect(callbacks[2][:name]).to eql :end_inline_image
@@ -923,22 +1058,10 @@ describe PDF::Reader, "integration specs" do
   context "PDF that uses a type3 bitmap font with a rare FontMatrix" do
     let(:filename) { pdf_spec_file("type3_font_with_rare_font_matrix") }
 
-    # TODO most type3 fonts have a FontMatrix entry of [ 0.001 0 0 0.001 0 0 ],
-    # which matches the glyph scale factor of 1000 that non-type3 fonts use.
-    # It's permitted for type3 fonts to use other FontMatrix values though,
-    # and we should do a better job of extracting the text.
-    # The Page is 200pts wide and 50pts high. The first letters for each word
-    # *should* be positioned like so:
-    #
-    #   P - X: 10.3 Y: 20   Width: 7.35 Height: 8.55
-    #   G - X: 56.5 Y: 19.7 Width: 8.25 Height: 9.15
-    #   A - X: 101.5 Y: 20  Width: 8.25 Height: 9
-    #
     it "extracts text correctly" do
-      pending
       PDF::Reader.open(filename) do |reader|
         page = reader.page(1)
-        expect(page.text).to include("Parallel Genetic Algorithms")
+        expect(page.text).to include("ParallelGenetic Algorithms")
       end
     end
   end
@@ -966,7 +1089,7 @@ describe PDF::Reader, "integration specs" do
     it "extracts text without raising an exception" do
       PDF::Reader.open(filename) do |reader|
         page = reader.page(1)
-        expect(page.text.split("\n").map(&:strip).slice(0,2)).to eq(["°","9"])
+        expect(page.text.split("\n").map(&:strip).slice(0,2)).to eq(["0","9"])
       end
     end
   end
@@ -982,6 +1105,17 @@ describe PDF::Reader, "integration specs" do
     end
   end
 
+  context "PDF with a TJ operator that aims to correct for character spacing" do
+    let(:filename) { pdf_spec_file("TJ_and_char_spacing") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text[15,17]).to eq("The big brown fox")
+      end
+    end
+  end
+
   context "PDF with a page that's missing the MediaBox attribute" do
     let(:filename) { pdf_spec_file("mediabox_missing") }
 
@@ -991,6 +1125,21 @@ describe PDF::Reader, "integration specs" do
         expect(page.text[0,54]).to eq("This page is missing the compulsory MediaBox attribute")
       end
     end
+
+    it "is portrait orientation" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.orientation).to eql("portrait")
+      end
+    end
+
+    it "returns correct page dimensions (defaults to portrait US letter)" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.width).to be_within(0.1).of(612)
+        expect(page.height).to be_within(0.1).of(792)
+      end
+    end
   end
 
   context "PDF using a standard fint and no difference table" do
@@ -1081,22 +1230,79 @@ describe PDF::Reader, "integration specs" do
     end
   end
 
-  context "Malformed PDF" do
-    let(:filename) { pdf_spec_file("trailer_root_is_not_a_dict") }
+  context "PDF with bad xref: using 1 not 0" do
+    let(:filename) { pdf_spec_file("minimal-xref-1") }
 
-    it "raises an exception if trailer Root is not a dict" do
+    it "extracts text correctly" do
       PDF::Reader.open(filename) do |reader|
-        expect { reader.page(1) }.to raise_error(PDF::Reader::MalformedPDFError)
+        page = reader.page(1)
+        expect(page.text).to eq("Hello World")
       end
     end
   end
 
-  context "PDF with missing page data" do
-    let(:filename) { pdf_spec_file("invalid_pages") }
+  context "PDF with octal data" do
+    let(:filename) { pdf_spec_file("octal101") }
 
-    it "raises a MalformedPDFError when an InvalidPageError is raised internally" do
+    it "extracts text correctly" do
       PDF::Reader.open(filename) do |reader|
-        expect { reader.pages }.to raise_error(PDF::Reader::MalformedPDFError)
+        page = reader.page(1)
+        expect(page.text).to eq("A ]A[")
+      end
+    end
+  end
+
+  context "PDF with octal data" do
+    let(:filename) { pdf_spec_file("octal74") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq("< ]<8[")
+      end
+    end
+  end
+
+  context "PDF with CR line-wrapped text" do
+    let(:filename) { pdf_spec_file("textwrapcr") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq("aaaa bbbb")
+      end
+    end
+  end
+
+  context "PDF with LF line-wrapped text" do
+    let(:filename) { pdf_spec_file("textwraplf") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq("aaaa bbbb")
+      end
+    end
+  end
+
+  context "PDF with CRLF line-wrapped text" do
+    let(:filename) { pdf_spec_file("textwrapcrlf") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq("aaaabbbb")
+      end
+    end
+  end
+
+  context "PDF with LFCR line-wrapped text" do
+    let(:filename) { pdf_spec_file("textwraplfcr") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq("aaaabbbb")
       end
     end
   end
@@ -1152,12 +1358,72 @@ describe PDF::Reader, "integration specs" do
         expect(page.text).to eq(text)
       end
     end
+
+    # This spec assumes high precision in our glyph positioning calcualtions. It's likely
+    # that the specific x,y co-ords tested here might change a bit over time as small bugs
+    # in glyph positioning are ironed out (kerning, etc). I still think this test is worthwhile,
+    # to confirm the positions don't change unintentionally.
+    #
+    # Pitstop numbers
+    #
+    # u -437.08, -68.018
+    # p -431.054, -70.4555
+    # s -424.851, -68.018
+    # i -420.4, -67.9
+    # d -416.918, -68.018
+    # e -410.978, -68.018
+    it "extracts individual chars with correct positions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        some_runs = page.runs(
+          merge: false,
+          rect: PDF::Reader::Rectangle.new(-438, -68, -405, -62)
+        )
+        expect(some_runs.size).to eql(6)
+        expect(some_runs[0].text).to eql("u")
+        expect(some_runs[0].origin).to be_close_to(PDF::Reader::Point.new(-437.24, -67.9))
+        expect(some_runs[1].text).to eql("p")
+        expect(some_runs[1].origin).to be_close_to(PDF::Reader::Point.new(-431.24, -67.9))
+        expect(some_runs[2].text).to eql("s")
+        expect(some_runs[2].origin).to be_close_to(PDF::Reader::Point.new(-425.34, -67.9))
+        expect(some_runs[3].text).to eql("i")
+        expect(some_runs[3].origin).to be_close_to(PDF::Reader::Point.new(-420.65, -67.9))
+        expect(some_runs[4].text).to eql("d")
+        expect(some_runs[4].origin).to be_close_to(PDF::Reader::Point.new(-417.35, -67.9))
+        expect(some_runs[5].text).to eql("e")
+        expect(some_runs[5].origin).to be_close_to(PDF::Reader::Point.new(-411.26, -67.9))
+      end
+    end
+
+    it "is portrait orientation" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.orientation).to eql("portrait")
+      end
+    end
+
+    it "returns correct page dimensions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        # A4 portrait
+        expect(page.width).to be_within(0.1).of(595.30)
+        expect(page.height).to be_within(0.1).of(841.88)
+      end
+    end
+
+    it "returns correct origin" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.origin.x).to be_within(0.1).of(-595.30)
+        expect(page.origin.y).to be_within(0.1).of(-841.88)
+      end
+    end
   end
 
-  context "PDF with page rotation followed by matrix transformations to undo it" do
+  context "PDF with page rotation of 270 degrees followed by matrix transformations to undo it" do
     let(:filename) { pdf_spec_file("rotate-then-undo") }
     let(:text) {
-      "This page uses matrix transformations to print text sideways, " +
+      "This page uses matrix transformations to print text   sideways, " +
       "then has a Rotate key to fix it"
     }
 
@@ -1167,5 +1433,352 @@ describe PDF::Reader, "integration specs" do
         expect(page.text).to eq(text)
       end
     end
+
+    # This spec assumes high precision in our glyph positioning calcualtions. It's likely
+    # that the specific x,y co-ords tested here might change a bit over time as small bugs
+    # in glyph positioning are ironed out (kerning, etc). I still think this test is worthwhile,
+    # to confirm the positions don't change unintentionally.
+    #
+    # Pitstop Numbers
+    #
+    # T -570.28, 750.95
+    # h -560.86, 750.95
+    # i -553.38, 750.95
+    # s -550.68, 750.76
+    #
+    it "extracts individual chars with correct positions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        some_runs = page.runs(
+          merge: false,
+          rect: PDF::Reader::Rectangle.new(-572, 749, -545, 760)
+        )
+        expect(some_runs.size).to eql(4)
+        expect(some_runs[0].text).to eql("T")
+        expect(some_runs[0].origin).to be_close_to(PDF::Reader::Point.new(-570.24, 750.95))
+        expect(some_runs[1].text).to eql("h")
+        expect(some_runs[1].origin).to be_close_to(PDF::Reader::Point.new(-561.95, 750.95))
+        expect(some_runs[2].text).to eql("i")
+        expect(some_runs[2].origin).to be_close_to(PDF::Reader::Point.new(-554.40, 750.95))
+        expect(some_runs[3].text).to eql("s")
+        expect(some_runs[3].origin).to be_close_to(PDF::Reader::Point.new(-551.39, 750.95))
+      end
+    end
+
+    it "is portrait orientation" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.orientation).to eql("portrait")
+      end
+    end
+
+    it "returns correct page dimensions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        # A4 portrait
+        expect(page.width).to be_within(0.1).of(595.30)
+        expect(page.height).to be_within(0.1).of(841.88)
+      end
+    end
+
+    it "returns correct origin" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.origin.x).to be_within(0.1).of(-595.30)
+        expect(page.origin.y).to be_within(0.1).of(0)
+      end
+    end
+  end
+
+  context "PDF with 270° page rotation and matrix transformations within BT block to undo" do
+    let(:filename) { pdf_spec_file("rotate-270-then-undo-inside-bt") }
+    let(:text) { "This page is rotated 270 degrees" }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq(text)
+      end
+    end
+
+    # This spec assumes high precision in our glyph positioning calcualtions. It's likely
+    # that the specific x,y co-ords tested here might change a bit over time as small bugs
+    # in glyph positioning are ironed out (kerning, etc). I still think this test is worthwhile,
+    # to confirm the positions don't change unintentionally.
+    #
+    # Pitstop numbers
+    #
+    # T -320.736, 534
+    # h -312.408, 534
+    # i -305.796, 534
+    # s -303.408, 533.832
+    #
+    it "extracts individual chars with correct positions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        some_runs = page.runs(
+          merge: false,
+          rect: PDF::Reader::Rectangle.new(-322, 534, -300, 550)
+        )
+        expect(some_runs.size).to eql(4)
+        expect(some_runs[0].text).to eql("T")
+        expect(some_runs[0].origin).to be_close_to(PDF::Reader::Point.new(-320.2, 534.5))
+        expect(some_runs[1].text).to eql("h")
+        expect(some_runs[1].origin).to be_close_to(PDF::Reader::Point.new(-312.86, 534.5))
+        expect(some_runs[2].text).to eql("i")
+        expect(some_runs[2].origin).to be_close_to(PDF::Reader::Point.new(-306.19, 534.5))
+        expect(some_runs[3].text).to eql("s")
+        expect(some_runs[3].origin).to be_close_to(PDF::Reader::Point.new(-303.532, 534.5))
+      end
+    end
+
+    it "is portrait orientation" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.orientation).to eql("portrait")
+      end
+    end
+
+    it "returns correct page dimensions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        # A4 portrait
+        expect(page.width).to be_within(0.1).of(595.30)
+        expect(page.height).to be_within(0.1).of(841.88)
+      end
+    end
+
+    it "returns correct origin" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.origin.x).to be_within(0.1).of(-595.30)
+        expect(page.origin.y).to be_within(0.1).of(0)
+      end
+    end
+  end
+
+  context "PDF with page rotation of 90 degrees followed by matrix transformations to undo it" do
+    let(:filename) { pdf_spec_file("rotate-90-then-undo") }
+    let(:text) {
+      "1: This PDF has Rotate:90 in the page metadata\n" +
+      "2: to get a landscape layout, and then uses matrix\n" +
+      "3: transformation to rotate the text back to normal"
+    }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to eq(text)
+      end
+    end
+
+    # This spec assumes high precision in our glyph positioning calcualtions. It's likely
+    # that the specific x,y co-ords tested here might change a bit over time as small bugs
+    # in glyph positioning are ironed out (kerning, etc). I still think this test is worthwhile,
+    # to confirm the positions don't change unintentionally.
+    #
+    # Pitstop numbers
+    #
+    # T 287.298, -45.49
+    # h 295.626, -45.49
+    # i 302.238, -45.49
+    # s 304.626, -45.658
+    #
+    it "extracts individual chars with correct positions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        some_runs = page.runs(
+          merge: false,
+          rect: PDF::Reader::Rectangle.new(285, -47, 310, -35)
+        )
+        expect(some_runs.size).to eql(4)
+        expect(some_runs[0].text).to eql("T")
+        expect(some_runs[0].origin).to be_close_to(PDF::Reader::Point.new(287.33, -45.49))
+        expect(some_runs[1].text).to eql("h")
+        expect(some_runs[1].origin).to be_close_to(PDF::Reader::Point.new(294.66, -45.49))
+        expect(some_runs[2].text).to eql("i")
+        expect(some_runs[2].origin).to be_close_to(PDF::Reader::Point.new(301.33, -45.49))
+        expect(some_runs[3].text).to eql("s")
+        expect(some_runs[3].origin).to be_close_to(PDF::Reader::Point.new(304, -45.49))
+      end
+    end
+
+    it "is portrait landscape" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.orientation).to eql("landscape")
+      end
+    end
+
+    it "returns correct page dimensions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        # A4 landscape
+        expect(page.width).to be_within(0.1).of(842)
+        expect(page.height).to be_within(0.1).of(595)
+      end
+    end
+
+    it "returns correct origin" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.origin.x).to be_within(0.1).of(0)
+        expect(page.origin.y).to be_within(0.1).of(-595)
+      end
+    end
+  end
+
+  context "PDF with page rotation of 90 degrees followed by matrix transformations to undo it" do
+    let(:filename) { pdf_spec_file("rotate-90-then-undo-with-br-text") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.text).to include("This PDF ha  sRotate:90 in the page")
+        expect(page.text).to include("metadata to get a landscape layout")
+        expect(page.text).to include("and text in bottom right quadrant")
+      end
+    end
+
+    # This spec assumes high precision in our glyph positioning calcualtions. It's likely
+    # that the specific x,y co-ords tested here might change a bit over time as small bugs
+    # in glyph positioning are ironed out (kerning, etc). I still think this test is worthwhile,
+    # to confirm the positions don't change unintentionally.
+    #
+    # Pitstop Numbers
+    #
+    # p 216.026, -525.906
+    # a 222.398, -523.578
+    # g 228.986, -525.906
+    # e 235.682, -523.578
+    #
+    it "extracts individual chars with correct positions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        some_runs = page.runs(
+          merge: false,
+          rect: PDF::Reader::Rectangle.new(212, -524, 239, -520)
+        )
+        expect(some_runs.size).to eql(4)
+        expect(some_runs[0].text).to eql("p")
+        expect(some_runs[0].origin).to be_close_to(PDF::Reader::Point.new(215.06, -523.41))
+        expect(some_runs[1].text).to eql("a")
+        expect(some_runs[1].origin).to be_close_to(PDF::Reader::Point.new(221.73, -523.41))
+        expect(some_runs[2].text).to eql("g")
+        expect(some_runs[2].origin).to be_close_to(PDF::Reader::Point.new(228.41, -523.41))
+        expect(some_runs[3].text).to eql("e")
+        expect(some_runs[3].origin).to be_close_to(PDF::Reader::Point.new(235.08, -523.41))
+      end
+    end
+
+    it "is landscape orientation" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.orientation).to eql("landscape")
+      end
+    end
+
+    it "returns correct page dimensions" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        # A4 landscape
+        expect(page.width).to be_within(0.1).of(842)
+        expect(page.height).to be_within(0.1).of(595)
+      end
+    end
+
+    it "returns correct origin" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.origin.x).to be_within(0.1).of(0)
+        expect(page.origin.y).to be_within(0.1).of(-595)
+      end
+    end
+  end
+
+  context "PDF with /Prev 0 trailer entry" do
+    let(:filename) { pdf_spec_file("prev0") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        expect(reader.page(1).text).to eql("aaaa bbbb")
+      end
+    end
+  end
+
+  context "Content stream with indirect object for filters array" do
+    let(:filename) { pdf_spec_file("stream-with-indirect-filters") }
+
+    it "extracts text correctly" do
+      PDF::Reader.open(filename) do |reader|
+        expect(reader.page(1).text).to eql(
+          "The content stream for this page stores the filter in an indirect object"
+        )
+      end
+    end
+  end
+
+  context "PDF with text outside the CropBox and MediaBox" do
+    let(:filename) { pdf_spec_file("text_outside_cropbox_and_mediabox") }
+
+    it "returns the correct rectangles for the page 1" do
+      PDF::Reader.open(filename) do |reader|
+        page = reader.page(1)
+        expect(page.rectangles[:MediaBox]).to eq(PDF::Reader::Rectangle.new(0, 0, 612, 792))
+        expect(page.rectangles[:CropBox]).to eq(PDF::Reader::Rectangle.new(100, 100, 412, 592))
+      end
+    end
+
+    it "by default only extracts text inside the CropBox" do
+      PDF::Reader.open(filename) do |reader|
+        text = reader.page(1).text
+        expect(text).to include("This text is inside the CropBox")
+        expect(text).to_not include("Between CropBox and MediaBox")
+        expect(text).to_not include("Outside MediaBox")
+      end
+    end
+
+    it "by default only extracts runs inside the CropBox" do
+      PDF::Reader.open(filename) do |reader|
+        text_from_runs = reader.page(1).runs.map(&:text).join(" ")
+        expect(text_from_runs).to include("This text is inside the CropBox")
+        expect(text_from_runs).to_not include("Between CropBox and MediaBox")
+        expect(text_from_runs).to_not include("Outside MediaBox")
+      end
+    end
+
+    it "can extract text between CropBox and MediaBox with a custom option" do
+      PDF::Reader.open(filename) do |reader|
+        text = reader.page(1).text(rect: PDF::Reader::Rectangle.new(0, 0, 612, 792))
+        expect(text).to include("This text is inside the CropBox")
+        expect(text).to include("Between CropBox and MediaBox")
+        expect(text).to_not include("Outside MediaBox")
+      end
+    end
+
+    it "can extract runs between CropBox and MediaBox with a custom option" do
+      mediabox = PDF::Reader::Rectangle.new(0, 0, 612, 792)
+      PDF::Reader.open(filename) do |reader|
+        text_from_runs = reader.page(1).runs(rect: mediabox).map(&:text).join(" ")
+        expect(text_from_runs).to include("This text is inside the CropBox")
+        expect(text_from_runs).to include("Between CropBox and MediaBox")
+        expect(text_from_runs).to_not include("Outside MediaBox")
+      end
+    end
+
+    # This currently doesn't work - PDF::Reader::PageLayout skips text that's outside the MediaBox
+    # I'm not sure we want it to work either, but I'm adding it as a pending spec for completeness
+    # while I consider the best options
+    it "can extract text outside the MediaBox with a custom option"
+
+    it "can extract runs outside the MediaBox with a custom option" do
+      larger_than_mediabox = PDF::Reader::Rectangle.new(0, 0, 900, 900)
+      PDF::Reader.open(filename) do |reader|
+        text_from_runs = reader.page(1).runs(rect: larger_than_mediabox).map(&:text).join(" ")
+        expect(text_from_runs).to include("This text is inside the CropBox")
+        expect(text_from_runs).to include("Between CropBox and MediaBox")
+        expect(text_from_runs).to include("Outside MediaBox")
+      end
+    end
   end
 end
